# $Id: Glossary.py 19160 2006-03-10 21:40:23Z schluep $
# Copyright (c) 2005, Samuel Schluep, ETH Zurich. All rights reserved.
# See also LICENSE.txt

# Zope
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
#import OFS
from OFS.SimpleItem import SimpleItem
# Silva
from Products.Silva.Content import Content
from Products.Silva.interfaces import IContent, IContainer, IPublication
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit, translateCdata
from Products.Silva import mangle
# Silva Glossary
from adapters.glossarizer import getGlossarizerAdapter


icon = "www/glossary.png"


class Glossary(Content, SimpleItem):
    """Glossaries can be created that function like a glossary in the back
       of a book. The glossary entries must be defined by the help of the
       Silva Glossary Entry content type and refered to using links in
       documents. Glossaries cascade downwards, searching for all glossary 
       entries in documents in the current and underlying folders and
       publications (note that only glossary entries in published
       documents will be listed).
    """
    security = ClassSecurityInfo()

    meta_type = "Silva Glossary"

    __implements__ = IContent

    def __init__(self, id, title):
        Glossary.inheritedAttribute('__init__')(self, id, title)
        # List of sorted Glossary Entry objects found. Items are tuples
        # with the glossary term and HTML rendered description.
        self._glossaryEntries = None

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_glossary_entries')
    def get_glossary_entries(self):
        """Returns a list of all glossary entries, sorted alphabetically.
        
        The items of the list are tuples conaining the glossary term and
        the glossary description rendered as HTML.
        """
        return self._glossaryEntries

    security.declareProtected(SilvaPermissions.ApproveSilvaContent,
                              'update')
    def update(self):
        """Update the glossary.
        
        Scan all glosarizable documents in the container and sub-containers
        where this glossary is located in. Updates the glossary internal
        list of glossary terms and their description rendered as HTML.
        """
        # Scan all objects in the container and add glossary entries to
        # the dictionary
        glossDict = {}
        for object in self._getGlossarizables():
            glossarizable = getGlossarizerAdapter(object)
            glossaryEntries = glossarizable.getGlossaryEntries()
            if not glossaryEntries:
                continue
            for glossaryEntry in glossaryEntries.keys():
                glossDict[glossaryEntry] = glossaryEntries[glossaryEntry]
        # copy glossary entries to a list and do case insensitive sort
        sorted_entries =[]
        for uid in glossDict.keys():
            entry = self.restrictedTraverse(uid)
            term = entry.get_title()
            term_lower = term.lower()
            # put description rendered as HTML into list
            supp = self.service_editorsupport.getMixedContentSupport(
            self, entry.description.documentElement)
            description = supp.renderHTML()
            sorted_entries.append( (term_lower, (term, description)) )
        sorted_entries.sort()
        # return a list with the glossary term and description
        entries = [entry for term_lower, entry in sorted_entries]
        self._glossaryEntries = entries

    def _getGlossarizables(self):
        """Gets a list of objects in the parent and sub-containers including
        the parent default object. Returns the list of Silva objects.
        """
        container = self.get_container()
        result = [container.get_default()] + \
                 [item for i, item in container.get_public_tree_all()]
        return result

    # XXX do we need this?
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'is_deletable')
    def is_deletable(self):
        """always deletable"""
        return 1

    # XXX do we need this?
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'can_set_title')
    def can_set_title(self):
        """return 1 so the title can be set"""
        return 1


InitializeClass(Glossary)


manage_addGlossaryForm = PageTemplateFile("www/glossaryAdd", globals(),
                                         __name__='manage_addGlossaryForm')

def manage_addGlossary(self, id, title, REQUEST=None):
    """Add a glossary."""
    if not mangle.Id(self, id).isValid():
        return
    object = Glossary(id, title)
    self._setObject(id, object)

    glossary = self._getOb(id)
    glossary.set_title(title)

    add_and_edit(self, id, REQUEST)
    return ''

